<?php
namespace PME\Controllers\RestricoesContratuais;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class FrequenciaPratoController {
    
  protected $entityManager;
  protected $newCode;
  protected $environment;
  protected $connection;
  
  public function __construct($entityManager, $connection, $newCode, $environment) {
  	$this->entityManager = $entityManager;
  	$this->newCode = $newCode;
  	$this->environment = $environment;
  	$this->connection = $connection;
  }
  
  public function returnCodFreqPrat(DTO\Request $request, DTO\Response $response) {
    $sql = "SELECT MAX(NRFREQPRATMESPAE) CODIGO FROM FREQPRATMESPAE";

    $codigo = $this->entityManager->getConnection()->fetchAll($sql);
    $response->addDataSet(new DataSet("codigo", $codigo));
  }
  
  public function returnCodFreqGrPrat(DTO\Request $request, DTO\Response $response) {
    $sql = "SELECT MAX(NRFREQGRPRATMESPAE) CODIGO FROM FREQGRPRATMESPAE";

    $codigo = $this->entityManager->getConnection()->fetchAll($sql);
    $response->addDataSet(new DataSet("codigo", $codigo));
  }
  
  public function getMaxItFreqPrat() {
    $sql = "SELECT MAX(NRITFREQPRATMESPAE) NRITFREQPRATMESPAE FROM ITFREQPRATMESPAE";

    $result = $this->entityManager->getConnection()->fetchAssoc($sql);
    return $result;
  }
  
  public function getMaxItFreqGrPrat() {
    $sql = "SELECT MAX(NRITFREQGRPRATMESPAE) NRITFREQGRPRATMESPAE FROM ITFREQGRPRATMESPAE";

    $result = $this->entityManager->getConnection()->fetchAssoc($sql);
    return $result;
  }
  
  private function validaPorPrato($param){
    // Valida se o prato já é cadastrado na restrição
		$sql = "SELECT I.NRITFREQPRATMESPAE, P.NMPRATO
                 FROM ITFREQPRATMESPAE I
                 INNER JOIN PRATO P ON P.CDPRATO = I.CDPRATO 
                 AND P.CDFILAUXPRAT = I.CDFILAUXPRAT
                WHERE I.CDPRATO = :CDPRATO
                  AND I.CDFILAUXPRAT = :CDFILAUXPRAT
                  AND I.NRFREQPRATMESPAE = :NRFREQPRATMESPAE
                   AND I.NRORG = :NRORG";
        
    $paramSql = array(
        'CDPRATO' => $param['CDPRATO'],
        'CDFILAUXPRAT' => $param['CDFILAUXPRAT'],
        'NRFREQPRATMESPAE' => $param['NRFREQPRATMESPAE'],
        'NRORG' => $this->environment->getNrOrg()
    );
    
    $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
    if (count($resultado) > 0) {
    	return 'ERROR_PRATO_CADASTRADO_VIG:' . $resultado[0]['NMPRATO'];
    }
    
    return true;
  }
  
  public function saveItFreqPrat(DTO\Request $request, DTO\Response $response) {
      try {			
    			$params = $request->getParameters()["editedRows"];
                $this->entityManager->getConnection()->beginTransaction();
    			
    			foreach ($params as $param) {
    			    if ($param['__is_new'] === true) {
    			        $validacao = $this->validaPorPrato($param);
					
        					if ($validacao !== true) {
        						$this->entityManager->getConnection()->rollBack();
        						$response->addDataset(new DataSet('mensagem', array($validacao)));
        						return;
        					};
    			      
        			    $nrItFrqPrat = $this->getMaxItFreqPrat();
        				
        				// Insere prato na frequencia de prato
        				$sql = "INSERT INTO ITFREQPRATMESPAE (NRITFREQPRATMESPAE, CDFILAUXPRAT, CDPRATO, NRMINFREQPRATMES,
        				            NRMAXFREQPRATMES, NRMAXFREQPRATSEM, NRFREQPRATMESPAE, NRORG)
        				        VALUES (:NRITFREQPRATMESPAE, :CDFILAUXPRAT, :CDPRATO, :NRMINFREQPRATMES, :NRMAXFREQPRATMES,
        				            :NRMAXFREQPRATSEM, :NRFREQPRATMESPAE, :NRORG)";
                        
                        $paramsIn = array(
                            'NRITFREQPRATMESPAE' => $nrItFrqPrat['NRITFREQPRATMESPAE'] + 1,
                            'CDFILAUXPRAT' => $param['CDFILAUXPRAT'],
                            'CDPRATO' => $param['CDPRATO'],
                            'NRMINFREQPRATMES' => $param['NRMINFREQPRATMES'],
                            'NRMAXFREQPRATMES' => $param['NRMAXFREQPRATMES'],
                            'NRMAXFREQPRATSEM' => $param['NRMAXFREQPRATSEM'],
                            'NRFREQPRATMESPAE' => $param['NRFREQPRATMESPAE'],
                            'NRORG' => $this->environment->getNrOrg()
                        );
                        
                        $this->entityManager->getConnection()->prepare($sql)->execute($paramsIn);
    			    } else {
    			        // Atualiza item de frequência existente
    			        $sql = "UPDATE ITFREQPRATMESPAE SET NRMINFREQPRATMES = :NRMINFREQPRATMES, NRMAXFREQPRATMES = :NRMAXFREQPRATMES,
    			                    NRMAXFREQPRATSEM = :NRMAXFREQPRATSEM
    			                 WHERE NRITFREQPRATMESPAE = :NRITFREQPRATMESPAE AND CDPRATO = :CDPRATO AND NRFREQPRATMESPAE = :NRFREQPRATMESPAE
    			                    AND NRORG = :NRORG";
                        
                        $paramsIn = array(
                            'NRITFREQPRATMESPAE' => $param['NRITFREQPRATMESPAE'],
                            'CDPRATO' => $param['CDPRATO'],
                            'NRMINFREQPRATMES' => $param['NRMINFREQPRATMES'],
                            'NRMAXFREQPRATMES' => $param['NRMAXFREQPRATMES'],
                            'NRMAXFREQPRATSEM' => $param['NRMAXFREQPRATSEM'],
                            'NRFREQPRATMESPAE' => $param['NRFREQPRATMESPAE'],
                            'NRORG' => $this->environment->getNrOrg()
                        );
                        
                        $this->entityManager->getConnection()->prepare($sql)->execute($paramsIn);
    			    }
    			}
			
            $this->entityManager->getConnection()->commit();
        }catch (\Exception $e) {
            $this->entityManager->getConnection()->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
  }
  
  private function validaPorGrupoPrato($params){
    // Valida se o prato já é cadastrado na restrição
		$sql = "SELECT I.NRITFREQGRPRATMESPAE, G.NMGRUPOPRATOPAE
                 FROM ITFREQGRPRATMESPAE I
               INNER JOIN GRUPOPRATOPAE G ON G.NRGRUPOPRATOPAE = I.NRGRUPOPRATOPAE
              INNER JOIN AGRUPGRPRAT A ON A.NRAGRUPGRPRAT = G.NRAGRUPGRPRAT
                WHERE I.NRFREQGRPRATMESPAE = :NRFREQGRPRATMESPAE
                 AND I.NRGRUPOPRATOPAE = :NRGRUPOPRATOPAE
                   AND I.NRORG = :NRORG";
        
    $paramSql = array(
        'NRFREQGRPRATMESPAE' => $params['NRFREQGRPRATMESPAE'],
        'NRGRUPOPRATOPAE' => $params['NRGRUPOPRATOPAE'],
        'NRORG' => $this->environment->getNrOrg()
    );
    
    $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
    if (count($resultado) > 0) {
    	return 'ERROR_GRUPO_CADASTRADO_VIG:' . $resultado[0]['NMGRUPOPRATOPAE'];
    }
    
    return true;
  }
  
  public function saveItFreqGrPrat(DTO\Request $request, DTO\Response $response) {
      try {			
  			$params = $request->getParameters()["editedRows"];
              $this->entityManager->getConnection()->beginTransaction();
  			
  			foreach ($params as $param) {
  			    if ($param['__is_new'] === true) {
  			         $validacao = $this->validaPorGrupoPrato($param);
					
        					if ($validacao !== true) {
        						$this->entityManager->getConnection()->rollBack();
        						$response->addDataset(new DataSet('mensagem', array($validacao)));
        						return;
        					};
      			    $nrItFrqPrat = $this->getMaxItFreqGrPrat();
      				
      				// Insere prato na frequencia de grupo de prato
      				$sql = "INSERT INTO ITFREQGRPRATMESPAE (NRITFREQGRPRATMESPAE, NRGRUPOPRATOPAE, NRMINFREQGRPRATMES, NRMAXFREQGRPRATMES,
      				            NRMAXFREQGRPRATSEM, NRFREQGRPRATMESPAE, NRORG)
      				        VALUES (:NRITFREQGRPRATMESPAE, :NRGRUPOPRATOPAE, :NRMINFREQGRPRATMES, :NRMAXFREQGRPRATMES, :NRMAXFREQGRPRATSEM,
      				            :NRFREQGRPRATMESPAE, :NRORG)";
                      
                      $paramsIn = array(
                          'NRITFREQGRPRATMESPAE' => $nrItFrqPrat['NRITFREQGRPRATMESPAE'] + 1,
                          'NRGRUPOPRATOPAE' => $param['NRGRUPOPRATOPAE'],
                          'NRMINFREQGRPRATMES' => $param['NRMINFREQGRPRATMES'],
                          'NRMAXFREQGRPRATMES' => $param['NRMAXFREQGRPRATMES'],
                          'NRMAXFREQGRPRATSEM' => $param['NRMAXFREQGRPRATSEM'],
                          'NRFREQGRPRATMESPAE' => $param['NRFREQGRPRATMESPAE'],
                          'NRORG' => $this->environment->getNrOrg()
                      );
                      
                      $this->entityManager->getConnection()->prepare($sql)->execute($paramsIn);
  			    } else {
  			        // Atualiza item de frequência de grupo de prato existente
  			        $sql = "UPDATE ITFREQGRPRATMESPAE SET NRMINFREQGRPRATMES = :NRMINFREQGRPRATMES, NRMAXFREQGRPRATMES = :NRMAXFREQGRPRATMES,
  			                    NRMAXFREQGRPRATSEM = :NRMAXFREQGRPRATSEM
  			                 WHERE NRITFREQGRPRATMESPAE = :NRITFREQGRPRATMESPAE AND NRGRUPOPRATOPAE = :NRGRUPOPRATOPAE AND NRFREQGRPRATMESPAE = :NRFREQGRPRATMESPAE
  			                    AND NRORG = :NRORG";
                      
                      $paramsIn = array(
                          'NRITFREQGRPRATMESPAE' => $param['NRITFREQGRPRATMESPAE'],
                          'NRGRUPOPRATOPAE' => $param['NRGRUPOPRATOPAE'],
                          'NRMINFREQGRPRATMES' => $param['NRMINFREQGRPRATMES'],
                          'NRMAXFREQGRPRATMES' => $param['NRMAXFREQGRPRATMES'],
                          'NRMAXFREQGRPRATSEM' => $param['NRMAXFREQGRPRATSEM'],
                          'NRFREQGRPRATMESPAE' => $param['NRFREQGRPRATMESPAE'],
                          'NRORG' => $this->environment->getNrOrg()
                      );
                      
                      $this->entityManager->getConnection()->prepare($sql)->execute($paramsIn);
    			    }
    			}
			
            $this->entityManager->getConnection()->commit();
        }catch (\Exception $e) {
            $this->entityManager->getConnection()->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
  }
}