<?php
/**
 * Created by PhpStorm.
 * User: phillipesilva
 * Date: 01/08/14
 * Time: 13:38
 */

namespace Folha\Logic\SituacaoFrequencia;


use Folha\Entities\PtoSitufreqparam;
use Folha\Entities\PtoSitufrequencia;
use Geral\Util\NovoCodigo;
use Setforms2\Shared\Registry;

class SituacaoFrequencia {
    public static function updateSituacaoFrequencia($dados)
    {
        $entityManager = Registry::getEntityManager();
        $db = $entityManager->getConnection();
        $db->beginTransaction();

        try
        {
            self::updateSituacao($dados);
            self::insertOrUpdateParametrosSituacao($dados);
            $db->commit();
        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }

    public static function updateSituacao($dados)
    {
        $situParam = PtoSitufrequencia::findOneByNrsitufrequenciaAndNrorg($dados['NRSITUFREQUENCIA'], $dados['NRORG']);

        $situParam->setNrsitufrequencia($dados['NRSITUFREQUENCIA']);
        $situParam->setDsabreviacao($dados['DSABREVIACAO']);
        $situParam->setDssitufrequencia($dados['DSSITUFREQUENCIA']);
        $situParam->setDtinivigencia($dados['DTINIVIGENCIA']);
        $situParam->setDtfimvigencia($dados['DTFIMVIGENCIA']);
        $situParam->setNradvertsitufreq($dados['NRADVERTSITUFREQ']);
        $situParam->setIdativo($dados['IDATIVO']);
        $situParam->setDtultatu($dados['DATAATUAL']);
        $situParam->setCdoperultatu($dados['CDOPERADOR']);
        $situParam->setNrorgultatu($dados['NRORG']);
        $situParam->persist();

        Registry::getEntityManager()->flush();
    }

    public static function insertOrUpdateParametrosSituacao($dados, $insert = null)
    {
        if($insert) {
            $entityManager = Registry::getEntityManager();
            $db = $entityManager->getConnection();
            $db->beginTransaction();
        }

        try {
            if($dados['NRORGTRAB'] == '' || is_null($dados['NRORGTRAB'])) {
                $dados['NRSITUFREQPARAM'] = NovoCodigo::geraCodigo($dados['NRORG'], 'PTO_SITUFREQPARAM', 1);
                $situParam = new PtoSitufreqparam();
                $situParam->setDtinclusao($dados['DATAATUAL']);
                $situParam->setNrorginclusao($dados['NRORG']);
                $situParam->setCdoperinclusao($dados['CDOPERADOR']);
            } else {
                $situParam = PtoSitufreqparam::findOneByNrsitufreqparamAndNrorg($dados['NRSITUFREQPARAM'], $dados['NRORG']);
            }

            $situParam->setNrsitufrequencia($dados['NRSITUFREQUENCIA']);
            $situParam->setNrsitufreqparam($dados['NRSITUFREQPARAM']);
            $situParam->setNrorg($dados['NRORG']);
            $situParam->setNreventom($dados['NREVENTOM']);
            $situParam->setIdgeraeventomesincid($dados['IDGERAEVENTOMESINCID']);
            $situParam->setIdperdedsr($dados['IDPERDEDSR']);
            $situParam->setIdperdedsrqtdifzero($dados['IDPERDEDSRQTDIFZERO']);
            $situParam->setQtocorreperdedsr($dados['QTOCORREPERDEDSR']);
            $situParam->setQtrefdiaperdedsr($dados['QTREFDIAPERDEDSR']);
            $situParam->setIdtipoqtdereferencia($dados['IDTIPOQTDEREFERENCIA']);
            $situParam->setIdtiposituacaofreq($dados['IDTIPOSITUACAOFREQ']);
            $situParam->setIdativo($dados['IDATIVO']);
            $situParam->setDtultatu($dados['DATAATUAL']);
            $situParam->setCdoperultatu($dados['CDOPERADOR']);
            $situParam->setNrorgultatu($dados['NRORG']);
            $situParam->persist();

            Registry::getEntityManager()->flush();

            if($insert) {
                $db->commit();
            }
        } catch (\Exception $e) {
            if($insert) {
                $db->rollback();
            }
            throw $e;
        }
    }

    public static function deleteParametrosFrequencia($nrSituFreq, $nrOrg)
    {
        $entityManager = Registry::getEntityManager();
        $db = $entityManager->getConnection();
        $db->beginTransaction();

        try {
            $situParam = PtoSitufreqparam::findOneByNrsitufrequenciaAndNrorg($nrSituFreq, $nrOrg);

            if(is_object($situParam)) {
                $situParam->delete();
            }

            $entityManager->flush();
            $db->commit();
        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }
}