import Vue from 'vue';
import Router from 'vue-router';
import {
  Config, Http, Loader, I18n, IDictionary, Router as RouterCore,
} from '@zeedhi/core';
import { ZdPage } from '@zeedhi/vue';
import { TekLibLocalStorage } from '@zeedhi/tek-lib';
import { LoadingService } from '@zeedhi/common';
import Store from './store';

Vue.use(Router);

const router = new Router({
  mode: 'history',
  base: process.env.BASE_URL,
  routes: [
    {
      path: '/',
      name: 'inicio',
      component: ZdPage,
      props: () => ({
        path: 'home',
        local: true,
      }),
    },
    {
      path: '/modules/:module/:page',
      name: 'microservice',
      component: () => import('./views/Microservice.vue'),
    },
    {
      path: '*',
      component: ZdPage,
      props: {
        path: 'notfound',
        local: true,
      },
    },
    {
      path: '/auth/login',
      name: 'login',
      component: ZdPage,
      props: () => ({
        path: 'login',
        local: true,
      }),
    },
    {
      path: '/ger21000_formas_apresentacao',
      name: 'formasDeApresentacao',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/parametrizacao/ger21000_formas_apresentacao',
        local: true,
      }),
    },
    {
      path: '/pme00600_elaboracao_cardapio',
      name: 'elaboracaoDoCardapio',
      component: ZdPage,
      props: () => ({
        path: '/pme/cardapio/pme00600_elaboracao_cardapio',
        local: true,
      }),
    },
    {
      path: '/pme06600_substituicao_de_pratos',
      name: 'subsPratosCardapio',
      component: ZdPage,
      props: () => ({
        path: '/pme/cardapio/pme06600_substituicao_de_pratos',
        local: true,
      }),
    },
    {
      path: '/ger96000_central',
      name: 'central',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/cadastro/ger96000_central',
        local: true,
      }),
    },
    {
      path: '/ger01800_unidade_de_medida',
      name: 'unidadeDeMedida',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/cadastro/ger01800_unidade_de_medida',
        local: true,
      }),
    },
    /* {
      path: '/pla01308_utilizacao_produto_receita',
      name: 'utilizacaoProdutos',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/parametrizacao/pla01308_utilizacao_produto_receita',
        local: true,
      }),
    }, */
    {
      path: '/pme11000_percentual_adesao_prato',
      name: 'percentualDeAdesaoDoPrato',
      component: ZdPage,
      props: () => ({
        path: '/pme/parametrizacao/pme11000_percentual_adesao_prato',
        local: true,
      }),
    },
    {
      path: '/pme00900_requisicao_servico_analitica',
      name: 'requisicaoPorServicoAnalitica',
      component: ZdPage,
      props: () => ({
        path: '/pme/requisicao/pme00900_requisicao_servico_analitica',
        local: true,
      }),
    },
    {
      path: '/pme00800_calculo_previsao_consumo',
      name: 'calculoDaPrevisaoDoConsumo',
      component: ZdPage,
      props: () => ({
        path: '/pme/requisicao/pme00800_calculo_previsao_consumo',
        local: true,
      }),
    },
    {
      path: '/pme03100_emissao_card_dia',
      name: 'emissaoDoCardapioDoDia',
      component: ZdPage,
      props: () => ({
        path: '/pme/relatorios/pme03100_emissao_card_dia',
        local: true,
      }),
    },
    {
      path: '/pme03000_emissao_card_periodo',
      name: 'emissaoDoCardapioDoPeriodo',
      component: ZdPage,
      props: () => ({
        path: '/pme/relatorios/pme03000_emissao_card_periodo',
        local: true,
      }),
    },
    {
      path: '/ger80500_nutricionista',
      name: 'nutricionista',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/parametrizacao/ger80500_nutricionista',
        local: true,
      }),
    },
    {
      path: '/pme00300_tipo_de_cardapio',
      name: 'tipoDeCardapio',
      component: ZdPage,
      props: () => ({
        path: '/pme/parametrizacao/pme00300_tipo_de_cardapio',
        local: true,
      }),
    },
    {
      path: '/pme00700_consumo_por_servico',
      name: 'consumoPorServico',
      component: ZdPage,
      props: () => ({
        path: '/pme/servico/pme00700_consumo_por_servico',
        local: true,
      }),
    },
    {
      path: '/pla01305_consulta_nutrientes',
      name: 'consultaNutrientes',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/prato/pla01305_consulta_nutrientes',
        local: true,
      }),
    },
    {
      path: '/pme00200_servico',
      name: 'servico',
      component: ZdPage,
      props: () => ({
        path: '/pme/servico/pme00200_servico',
        local: true,
      }),
    },
    {
      path: '/report',
      name: 'report',
      component: ZdPage,
      props: () => ({
        path: 'report',
        local: true,
      }),
    },
    {
      path: '/pme01000_emissao_card_semana',
      name: 'emissaoDoCardapioDaSemana',
      component: ZdPage,
      props: () => ({
        path: '/pme/relatorios/pme01000_emissao_card_semana',
        local: true,
      }),
    },
    {
      path: '/pme02400_clientes_por_servico',
      name: 'clientesPorServico',
      component: ZdPage,
      props: () => ({
        path: '/pme/servico/pme02400_clientes_por_servico',
        local: true,
      }),
    },
    {
      path: '/pme09100_nutriente_cardapio',
      name: 'nutrientesDoCardapio',
      component: ZdPage,
      props: () => ({
        path: '/pme/relatorios/pme09100_nutriente_cardapio',
        local: true,
      }),
    },
    {
      path: '/pme00500_comensais_estimados',
      name: 'comensaisEstimados',
      component: ZdPage,
      props: () => ({
        path: '/pme/parametrizacao/pme00500_comensais_estimados',
        local: true,
      }),
    },
    {
      path: '/pme08100_custo_real_periodo',
      name: 'custoRealPeriodo',
      component: ZdPage,
      props: () => ({
        path: '/pme/custo/pme08100_custo_real_periodo',
        local: true,
      }),
    },
    {
      path: '/pme01300_emissao_card_periodo_PAE',
      name: 'emissaoDoCardapioDoPeriodoPAE',
      component: ZdPage,
      props: () => ({
        path: '/pme/relatorios/pme01300_emissao_card_periodo_PAE',
        local: true,
      }),
    },
    {
      path: '/pme08000_custo_planejado_cardapio',
      name: 'custoPlanejadoDoCardapio',
      component: ZdPage,
      props: () => ({
        path: '/pme/custo/pme08000_custo_planejado_cardapio',
        local: true,
      }),
    },
    {
      path: '/pme01400_efetivos',
      name: 'efetivos',
      component: ZdPage,
      props: () => ({
        path: '/pme/medicao/pme01400_efetivos',
        local: true,
      }),
    },
    {
      path: '/ger02500_nutrientes',
      name: 'nutrientes',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/parametrizacao/ger02500_nutrientes',
        local: true,
      }),
    },
    {
      path: '/ger04000_nutrientes_por_produto',
      name: 'nutrientesPorProd',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/parametrizacao/ger04000_nutrientes_por_produto',
        local: true,
      }),
    },
    {
      path: '/ger30000_padrao_de_servico',
      name: 'padraoDeServico',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/servico/ger30000_padrao_de_servico',
        local: true,
      }),
    },
    {
      path: '/pme02000_retirada_inventario',
      name: 'retiradaDoMerendaComBaseNoInventario',
      component: ZdPage,
      props: () => ({
        path: '/pme/medicao/pme02000_retirada_inventario',
        local: true,
      }),
    },
    {
      path: '/pme01600_desvio_de_consumo',
      name: 'desvioConsumo',
      component: ZdPage,
      props: () => ({
        path: '/pme/custo/pme01600_desvio_de_consumo',
        local: true,
      }),
    },
    {
      path: '/pme01700_custo_realizado_global_do_periodo',
      name: 'custoRealizadoGlobalDoPeriodo',
      component: ZdPage,
      props: () => ({
        path: '/pme/custo/pme01700_custo_realizado_global_do_periodo',
        local: true,
      }),
    },
    {
      path: '/pme01100_custo_planejado_realizado',
      name: 'custoPlanejadoRealizado',
      component: ZdPage,
      props: () => ({
        path: '/pme/custo/pme01100_custo_planejado_realizado',
        local: true,
      }),
    },
    {
      path: '/pme01800_consumo_planejado_realizado',
      name: 'consumoPlanejadoRealizado',
      component: ZdPage,
      props: () => ({
        path: '/pme/custo/pme01800_consumo_planejado_realizado',
        local: true,
      }),
    },
    {
      path: '/pme08200_custo_card_mes',
      name: 'custoCardapioMes',
      component: ZdPage,
      props: () => ({
        path: '/pme/custo/pme08200_custo_card_mes',
        local: true,
      }),
    },
    {
      path: '/ger28000_tipo_de_retirada',
      name: 'tipoDeRetirada',
      component: ZdPage,
      props: () => ({
        path: 'planejamento/parametrizacao/ger28000_tipo_de_retirada',
        local: true,
      }),
    },
    {
      path: '/pla01900_parametros_de_planejamento',
      name: 'parametrosPlanejamento',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/parametrizacao/pla01900_parametros_de_planejamento',
        local: true,
      }),
    },
    {
      path: '/pla01303_emissao_receitas',
      name: 'plaEmissaoReceitas',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/prato/pla01303_emissao_receitas',
        local: true,
      }),
    },
    {
      path: '/pla01400_tipo_de_prato',
      name: 'tipoDePrato',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/prato/pla01400_tipo_de_prato',
        local: true,
      }),
    },
    {
      path: '/pla00470_embalagem',
      name: 'embalagem',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/cadastro/pla00470_embalagem',
        local: true,
      }),
    },
    {
      path: '/pla15300_categoria_embalagens',
      name: 'categoria',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/cadastro/pla15300_categoria_embalagens',
        local: true,
      }),
    },
    {
      path: '/dih52300_medicao_caseira',
      name: 'medicaoCaseira',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/parametrizacao/dih52300_medicao_caseira',
        local: true,
      }),
    },
    {
      path: '/pla15100_formas_de_preparo',
      name: 'formasDePreparo',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/parametrizacao/pla15100_formas_de_preparo',
        local: true,
      }),
    },
    {
      path: '/ger01000_unidade_adicional_produto',
      name: 'unidadeAdicionalDoProduto',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/produto/ger01000_unidade_adicional_produto',
        local: true,
      }),
    },
    {
      path: '/ger22000_apresentacao_produto',
      name: 'apresentacaoDoProduto',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/produto/ger22000_apresentacao_produto',
        local: true,
      }),
    },
    {
      path: '/pla01300_prato',
      name: 'prato',
      component: ZdPage,
      props: () => ({
        path: '/planejamento/prato/pla01300_prato',
        local: true,
      }),
    },
    {
      path: '/pla13200_agrupamento_grupo_pratos',
      name: 'agrupamentoGrupoPratos',
      component: ZdPage,
      props: () => ({
        path: '/pme/restricoesContratuais/pla13200_agrupamento_grupo_pratos',
        local: true,
      }),
    },
    {
      path: '/pla13100_padrao_grupo_prato',
      name: 'padraoGrupoPrato',
      component: ZdPage,
      props: () => ({
        path: '/pme/restricoesContratuais/pla13100_padrao_grupo_prato',
        local: true,
      }),
    },
    {
      path: '/pme12100_freq_pratos_periodo',
      name: 'restricoesContratuais',
      component: ZdPage,
      props: () => ({
        path: '/pme/restricoesContratuais/pme12100_freq_pratos_periodo',
        local: true,
      }),
    },
    {
      path: '/pme12000_grupo_prato',
      name: 'grupoPrato',
      component: ZdPage,
      props: () => ({
        path: '/pme/restricoesContratuais/pme12000_grupo_prato',
        local: true,
      }),
    },
  ],
});

const isOpeningNewModule = function isOpeningNewModule(to: any, from: any) {
  const isFromProduct = (to.name === 'microservice' && from.name !== 'microservice');
  // eslint-disable-next-line max-len
  const isFromAnotherModule = ((to.name === 'microservice' && from.name === 'microservice') && to.path.split('/')[2] !== from.path.split('/')[2]);
  const isNotLoginRoute = (to.fullPath !== Config.env.modules.login.routePath);
  return ((isFromProduct || isFromAnotherModule) && isNotLoginRoute);
};

// eslint-disable-next-line max-len
const callLibStartSessionFromModule = async function callLibStartSessionFromModule(to: any) {
  const data = {
    USER: localStorage.getItem('LOGIN_USER'),
    LOGIN_LANGUAGE: localStorage.getItem('LOGIN_LANGUAGE'),
    LOGIN_TOKEN: localStorage.getItem('LOGIN_TOKEN'),
  };
  await Http.post(`${Config.env.modules[to.path.split('/')[2]].endPoint}/lib_startSession`, data);
};

router.beforeEach(async (to, from, next) => {
  if (isOpeningNewModule(to, from)) {
    try {
      LoadingService.show();
      await callLibStartSessionFromModule(to);
      LoadingService.hide();
    } catch (e) {
      LoadingService.hide();
    }
  }

  TekLibLocalStorage.setItem('IS_NEXT', true);

  if (to.path !== Config.env.loginRoutePath && !localStorage.getItem('LOGIN_TOKEN')) {
    next(Config.env.loginRoutePath);
  } else if (to.path === Config.env.loginRoutePath && localStorage.getItem('LOGIN_TOKEN')) {
    next(Config.env.baseUrl);
  } else {
    next();
  }
});

router.afterEach((to, from) => {
  const htmlElement = document.getElementsByTagName('html')[0];
  const appController = Loader.getInstance('AppController');
  if (to.name === 'microservice') {
    htmlElement.setAttribute('microservice', '');
    Store.commit('setRouterParams', to.params);
    const frame = (document.getElementById('microservice-frame') as HTMLIFrameElement);
    if (frame && frame.contentWindow && to.path.split('/')[2] === from.path.split('/')[2]) {
      frame.contentWindow.postMessage({
        type: 'microservice-route-change',
        routerParams: to.params,
      }, `${Config.env.modules[to.path.split('/')[2]].baseUrl}`);
    }
    window.document.title = `${Config.env.product.name} - ${I18n.translate('pme::login')}`;
  } else {
    htmlElement.removeAttribute('microservice');
    window.document.title = `${Config.env.product.name} - ${I18n.translate(`pme::${to.name}`)}`;
  }

  if (to.name === 'login') {
    htmlElement.setAttribute('login', '');
  } else {
    htmlElement.removeAttribute('login');
  }
  
  const currentRoute = RouterCore.getCurrentRoute() as IDictionary;
  const loginRoute = Config.env.loginRoutePath;
  
  if (currentRoute.path !== loginRoute) {
    if (appController.isNotMini) appController.routerAcessMini();
  }
  
  appController.centralIsValid();
});

export default router;
